# This file was auto-generated by Fern from our API Definition.

import contextlib
import typing
from json.decoder import JSONDecodeError

from .. import core
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.unchecked_base_model import construct_type
from ..errors.forbidden_error import ForbiddenError
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.allowed_output_formats import AllowedOutputFormats
from ..types.music_upload_response import MusicUploadResponse
from .types.body_compose_music_v_1_music_post_composition_plan import BodyComposeMusicV1MusicPostCompositionPlan
from .types.body_compose_music_v_1_music_post_model_id import BodyComposeMusicV1MusicPostModelId
from .types.body_compose_music_with_a_detailed_response_v_1_music_detailed_post_composition_plan import (
    BodyComposeMusicWithADetailedResponseV1MusicDetailedPostCompositionPlan,
)
from .types.body_compose_music_with_a_detailed_response_v_1_music_detailed_post_model_id import (
    BodyComposeMusicWithADetailedResponseV1MusicDetailedPostModelId,
)
from .types.body_stream_composed_music_v_1_music_stream_post_composition_plan import (
    BodyStreamComposedMusicV1MusicStreamPostCompositionPlan,
)
from .types.body_stream_composed_music_v_1_music_stream_post_model_id import (
    BodyStreamComposedMusicV1MusicStreamPostModelId,
)
from .types.music_compose_detailed_request_output_format import MusicComposeDetailedRequestOutputFormat
from .types.music_compose_request_output_format import MusicComposeRequestOutputFormat
from .types.music_separate_stems_request_stem_variation_id import MusicSeparateStemsRequestStemVariationId
from .types.music_stream_request_output_format import MusicStreamRequestOutputFormat
from .types.music_video_to_music_request_model_id import MusicVideoToMusicRequestModelId

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawMusicClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    @contextlib.contextmanager
    def video_to_music(
        self,
        *,
        videos: typing.List[core.File],
        output_format: typing.Optional[AllowedOutputFormats] = None,
        description: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.List[str]] = OMIT,
        model_id: typing.Optional[MusicVideoToMusicRequestModelId] = OMIT,
        sign_with_c_2_pa: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[HttpResponse[typing.Iterator[bytes]]]:
        """
        Generate background music from one or more video files. Videos are combined in order. Optional description and style tags influence the generated music.

        Parameters
        ----------
        videos : typing.List[core.File]
            See core.File for more documentation

        output_format : typing.Optional[AllowedOutputFormats]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. So an mp3 with 22.05kHz sample rate at 32kbs is represented as mp3_22050_32. MP3 with 192kbps bitrate requires you to be subscribed to Creator tier or above. PCM with 44.1kHz sample rate requires you to be subscribed to Pro tier or above. Note that the μ-law format (sometimes written mu-law, often approximated as u-law) is commonly used for Twilio audio inputs.

        description : typing.Optional[str]
            Optional text description of the music you want. A maximum of 1000 characters is allowed.

        tags : typing.Optional[typing.List[str]]
            Optional list of style tags (e.g. ['upbeat', 'cinematic']). A maximum of 10 tags is allowed.

        model_id : typing.Optional[MusicVideoToMusicRequestModelId]
            The model to use for the generation.

        sign_with_c_2_pa : typing.Optional[bool]
            Whether to sign the generated song with C2PA. Applicable only for mp3 files.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[HttpResponse[typing.Iterator[bytes]]]
            Generated audio file matching the video. Content-Type and file extension depend on the output_format parameter (default mp3).
        """
        with self._client_wrapper.httpx_client.stream(
            "v1/music/video-to-music",
            method="POST",
            params={
                "output_format": output_format,
            },
            data={
                "description": description,
                "tags": tags,
                "model_id": model_id,
                "sign_with_c2pa": sign_with_c_2_pa,
            },
            files={
                "videos": videos,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        ) as _response:

            def _stream() -> HttpResponse[typing.Iterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", 1024) if request_options is not None else 1024
                        return HttpResponse(
                            response=_response, data=(_chunk for _chunk in _response.iter_bytes(chunk_size=_chunk_size))
                        )
                    _response.read()
                    if _response.status_code == 403:
                        raise ForbiddenError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 422:
                        raise UnprocessableEntityError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield _stream()

    @contextlib.contextmanager
    def compose(
        self,
        *,
        output_format: typing.Optional[MusicComposeRequestOutputFormat] = None,
        prompt: typing.Optional[str] = OMIT,
        composition_plan: typing.Optional[BodyComposeMusicV1MusicPostCompositionPlan] = OMIT,
        music_length_ms: typing.Optional[int] = OMIT,
        model_id: typing.Optional[BodyComposeMusicV1MusicPostModelId] = OMIT,
        seed: typing.Optional[int] = OMIT,
        force_instrumental: typing.Optional[bool] = OMIT,
        respect_sections_durations: typing.Optional[bool] = OMIT,
        store_for_inpainting: typing.Optional[bool] = OMIT,
        sign_with_c_2_pa: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[HttpResponse[typing.Iterator[bytes]]]:
        """
        Compose a song from a prompt or a composition plan.

        Parameters
        ----------
        output_format : typing.Optional[MusicComposeRequestOutputFormat]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. Use "auto" (the default) to let the API pick the best format for the selected model: mp3_44100_128 for v1 models and mp3_48000_192 for v2 models.

        prompt : typing.Optional[str]
            A simple text prompt to generate a song from. Cannot be used in conjunction with `composition_plan`.

        composition_plan : typing.Optional[BodyComposeMusicV1MusicPostCompositionPlan]
            A detailed composition plan to guide music generation. Cannot be used in conjunction with `prompt`.

        music_length_ms : typing.Optional[int]
            The length of the song to generate in milliseconds. Used only in conjunction with `prompt`. Must be between 3000ms and 600000ms. Optional - if not provided, the model will choose a length based on the prompt.

        model_id : typing.Optional[BodyComposeMusicV1MusicPostModelId]
            The model to use for the generation.

        seed : typing.Optional[int]
            Random seed to initialize the music generation process. Providing the same seed with the same parameters can help achieve more consistent results, but exact reproducibility is not guaranteed and outputs may change across system updates. Cannot be used in conjunction with prompt.

        force_instrumental : typing.Optional[bool]
            If true, guarantees that the generated song will be instrumental. If false, the song may or may not be instrumental depending on the `prompt`. Can only be used with `prompt`.

        respect_sections_durations : typing.Optional[bool]
            Controls how strictly section durations in the `composition_plan` are enforced. Only used with `composition_plan` and only applies to `music_v1`; for `music_v2` section durations are always enforced and this is ignored. When false for `music_v1`, the model may adjust individual section durations for better quality and latency, while preserving the total song duration from the plan.

        store_for_inpainting : typing.Optional[bool]
            Whether to store the generated song for inpainting.

        sign_with_c_2_pa : typing.Optional[bool]
            Whether to sign the generated song with C2PA. Applicable only for mp3 files.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[HttpResponse[typing.Iterator[bytes]]]
            The generated audio file in the format specified
        """
        with self._client_wrapper.httpx_client.stream(
            "v1/music",
            method="POST",
            params={
                "output_format": output_format,
            },
            json={
                "prompt": prompt,
                "composition_plan": convert_and_respect_annotation_metadata(
                    object_=composition_plan, annotation=BodyComposeMusicV1MusicPostCompositionPlan, direction="write"
                ),
                "music_length_ms": music_length_ms,
                "model_id": model_id,
                "seed": seed,
                "force_instrumental": force_instrumental,
                "respect_sections_durations": respect_sections_durations,
                "store_for_inpainting": store_for_inpainting,
                "sign_with_c2pa": sign_with_c_2_pa,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:

            def _stream() -> HttpResponse[typing.Iterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", 1024) if request_options is not None else 1024
                        return HttpResponse(
                            response=_response, data=(_chunk for _chunk in _response.iter_bytes(chunk_size=_chunk_size))
                        )
                    _response.read()
                    if _response.status_code == 422:
                        raise UnprocessableEntityError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield _stream()

    @contextlib.contextmanager
    def compose_detailed(
        self,
        *,
        output_format: typing.Optional[MusicComposeDetailedRequestOutputFormat] = None,
        prompt: typing.Optional[str] = OMIT,
        composition_plan: typing.Optional[
            BodyComposeMusicWithADetailedResponseV1MusicDetailedPostCompositionPlan
        ] = OMIT,
        music_length_ms: typing.Optional[int] = OMIT,
        model_id: typing.Optional[BodyComposeMusicWithADetailedResponseV1MusicDetailedPostModelId] = OMIT,
        seed: typing.Optional[int] = OMIT,
        force_instrumental: typing.Optional[bool] = OMIT,
        respect_sections_durations: typing.Optional[bool] = OMIT,
        store_for_inpainting: typing.Optional[bool] = OMIT,
        with_timestamps: typing.Optional[bool] = OMIT,
        sign_with_c_2_pa: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[HttpResponse[typing.Iterator[bytes]]]:
        """
        Compose a song from a prompt or a composition plan.

        Parameters
        ----------
        output_format : typing.Optional[MusicComposeDetailedRequestOutputFormat]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. Use "auto" (the default) to let the API pick the best format for the selected model: mp3_44100_128 for v1 models and mp3_48000_192 for v2 models.

        prompt : typing.Optional[str]
            A simple text prompt to generate a song from. Cannot be used in conjunction with `composition_plan`.

        composition_plan : typing.Optional[BodyComposeMusicWithADetailedResponseV1MusicDetailedPostCompositionPlan]
            A detailed composition plan to guide music generation. Cannot be used in conjunction with `prompt`.

        music_length_ms : typing.Optional[int]
            The length of the song to generate in milliseconds. Used only in conjunction with `prompt`. Must be between 3000ms and 600000ms. Optional - if not provided, the model will choose a length based on the prompt.

        model_id : typing.Optional[BodyComposeMusicWithADetailedResponseV1MusicDetailedPostModelId]
            The model to use for the generation.

        seed : typing.Optional[int]
            Random seed to initialize the music generation process. Providing the same seed with the same parameters can help achieve more consistent results, but exact reproducibility is not guaranteed and outputs may change across system updates. Cannot be used in conjunction with prompt.

        force_instrumental : typing.Optional[bool]
            If true, guarantees that the generated song will be instrumental. If false, the song may or may not be instrumental depending on the `prompt`. Can only be used with `prompt`.

        respect_sections_durations : typing.Optional[bool]
            Controls how strictly section durations in the `composition_plan` are enforced. Only used with `composition_plan` and only applies to `music_v1`; for `music_v2` section durations are always enforced and this is ignored. When false for `music_v1`, the model may adjust individual section durations for better quality and latency, while preserving the total song duration from the plan.

        store_for_inpainting : typing.Optional[bool]
            Whether to store the generated song for inpainting.

        with_timestamps : typing.Optional[bool]
            Whether to return the timestamps of the words in the generated song.

        sign_with_c_2_pa : typing.Optional[bool]
            Whether to sign the generated song with C2PA. Applicable only for mp3 files.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[HttpResponse[typing.Iterator[bytes]]]
            Multipart/mixed response with JSON metadata and binary audio file
        """
        with self._client_wrapper.httpx_client.stream(
            "v1/music/detailed",
            method="POST",
            params={
                "output_format": output_format,
            },
            json={
                "prompt": prompt,
                "composition_plan": convert_and_respect_annotation_metadata(
                    object_=composition_plan,
                    annotation=BodyComposeMusicWithADetailedResponseV1MusicDetailedPostCompositionPlan,
                    direction="write",
                ),
                "music_length_ms": music_length_ms,
                "model_id": model_id,
                "seed": seed,
                "force_instrumental": force_instrumental,
                "respect_sections_durations": respect_sections_durations,
                "store_for_inpainting": store_for_inpainting,
                "with_timestamps": with_timestamps,
                "sign_with_c2pa": sign_with_c_2_pa,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:

            def _stream() -> HttpResponse[typing.Iterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", 1024) if request_options is not None else 1024
                        return HttpResponse(
                            response=_response, data=(_chunk for _chunk in _response.iter_bytes(chunk_size=_chunk_size))
                        )
                    _response.read()
                    if _response.status_code == 422:
                        raise UnprocessableEntityError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield _stream()

    @contextlib.contextmanager
    def stream(
        self,
        *,
        output_format: typing.Optional[MusicStreamRequestOutputFormat] = None,
        prompt: typing.Optional[str] = OMIT,
        composition_plan: typing.Optional[BodyStreamComposedMusicV1MusicStreamPostCompositionPlan] = OMIT,
        music_length_ms: typing.Optional[int] = OMIT,
        model_id: typing.Optional[BodyStreamComposedMusicV1MusicStreamPostModelId] = OMIT,
        seed: typing.Optional[int] = OMIT,
        force_instrumental: typing.Optional[bool] = OMIT,
        store_for_inpainting: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[HttpResponse[typing.Iterator[bytes]]]:
        """
        Stream a composed song from a prompt or a composition plan.

        Parameters
        ----------
        output_format : typing.Optional[MusicStreamRequestOutputFormat]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. Use "auto" (the default) to let the API pick the best format for the selected model: mp3_44100_128 for v1 models and mp3_48000_192 for v2 models.

        prompt : typing.Optional[str]
            A simple text prompt to generate a song from. Cannot be used in conjunction with `composition_plan`.

        composition_plan : typing.Optional[BodyStreamComposedMusicV1MusicStreamPostCompositionPlan]
            A detailed composition plan to guide music generation. Cannot be used in conjunction with `prompt`.

        music_length_ms : typing.Optional[int]
            The length of the song to generate in milliseconds. Used only in conjunction with `prompt`. Must be between 3000ms and 600000ms. Optional - if not provided, the model will choose a length based on the prompt.

        model_id : typing.Optional[BodyStreamComposedMusicV1MusicStreamPostModelId]
            The model to use for the generation.

        seed : typing.Optional[int]
            Random seed to initialize the music generation process. Providing the same seed with the same parameters can help achieve more consistent results, but exact reproducibility is not guaranteed and outputs may change across system updates. Cannot be used in conjunction with prompt.

        force_instrumental : typing.Optional[bool]
            If true, guarantees that the generated song will be instrumental. If false, the song may or may not be instrumental depending on the `prompt`. Can only be used with `prompt`.

        store_for_inpainting : typing.Optional[bool]
            Whether to store the generated song for inpainting.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[HttpResponse[typing.Iterator[bytes]]]
            Streaming audio data in the format specified
        """
        with self._client_wrapper.httpx_client.stream(
            "v1/music/stream",
            method="POST",
            params={
                "output_format": output_format,
            },
            json={
                "prompt": prompt,
                "composition_plan": convert_and_respect_annotation_metadata(
                    object_=composition_plan,
                    annotation=BodyStreamComposedMusicV1MusicStreamPostCompositionPlan,
                    direction="write",
                ),
                "music_length_ms": music_length_ms,
                "model_id": model_id,
                "seed": seed,
                "force_instrumental": force_instrumental,
                "store_for_inpainting": store_for_inpainting,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:

            def _stream() -> HttpResponse[typing.Iterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", 1024) if request_options is not None else 1024
                        return HttpResponse(
                            response=_response, data=(_chunk for _chunk in _response.iter_bytes(chunk_size=_chunk_size))
                        )
                    _response.read()
                    if _response.status_code == 422:
                        raise UnprocessableEntityError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield _stream()

    def upload(
        self,
        *,
        file: core.File,
        extract_composition_plan: typing.Optional[str] = OMIT,
        with_timestamps: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[MusicUploadResponse]:
        """
        Upload a music file to be later used for inpainting. Price for uploading is the same as the one for song generation. All uploaded content gets inspected for copyright infringement. If copyrighted content is detected, half of the request cost is still charged.

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        extract_composition_plan : typing.Optional[str]
            Whether to generate and return the composition plan for the uploaded song. Pass a model id (`music_v1` or `music_v2`) to control which composition plan format is returned. Passing `true`/`false` is deprecated; `true` defaults to the `music_v1` plan format. Enabling this will increase the latency.

        with_timestamps : typing.Optional[bool]
            Whether to transcribe the uploaded song and return word-level timestamps. If True, the response will include words_timestamps but will increase the latency.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[MusicUploadResponse]
            Successfully uploaded music file with optional composition plan
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/music/upload",
            method="POST",
            data={
                "extract_composition_plan": extract_composition_plan,
                "with_timestamps": with_timestamps,
            },
            files={
                "file": file,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MusicUploadResponse,
                    construct_type(
                        type_=MusicUploadResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    @contextlib.contextmanager
    def separate_stems(
        self,
        *,
        file: core.File,
        output_format: typing.Optional[AllowedOutputFormats] = None,
        stem_variation_id: typing.Optional[MusicSeparateStemsRequestStemVariationId] = OMIT,
        sign_with_c_2_pa: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[HttpResponse[typing.Iterator[bytes]]]:
        """
        Separate an audio file into individual stems. This endpoint might have high latency, depending on the length of the audio file.

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        output_format : typing.Optional[AllowedOutputFormats]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. So an mp3 with 22.05kHz sample rate at 32kbs is represented as mp3_22050_32. MP3 with 192kbps bitrate requires you to be subscribed to Creator tier or above. PCM with 44.1kHz sample rate requires you to be subscribed to Pro tier or above. Note that the μ-law format (sometimes written mu-law, often approximated as u-law) is commonly used for Twilio audio inputs.

        stem_variation_id : typing.Optional[MusicSeparateStemsRequestStemVariationId]
            The id of the stem variation to use.

        sign_with_c_2_pa : typing.Optional[bool]
            Whether to sign the generated song with C2PA. Applicable only for mp3 files.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[HttpResponse[typing.Iterator[bytes]]]
            ZIP archive containing separated audio stems. Each stem is provided as a separate audio file in the requested output format.
        """
        with self._client_wrapper.httpx_client.stream(
            "v1/music/stem-separation",
            method="POST",
            params={
                "output_format": output_format,
            },
            data={
                "stem_variation_id": stem_variation_id,
                "sign_with_c2pa": sign_with_c_2_pa,
            },
            files={
                "file": file,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        ) as _response:

            def _stream() -> HttpResponse[typing.Iterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", 1024) if request_options is not None else 1024
                        return HttpResponse(
                            response=_response, data=(_chunk for _chunk in _response.iter_bytes(chunk_size=_chunk_size))
                        )
                    _response.read()
                    if _response.status_code == 422:
                        raise UnprocessableEntityError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield _stream()


class AsyncRawMusicClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    @contextlib.asynccontextmanager
    async def video_to_music(
        self,
        *,
        videos: typing.List[core.File],
        output_format: typing.Optional[AllowedOutputFormats] = None,
        description: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.List[str]] = OMIT,
        model_id: typing.Optional[MusicVideoToMusicRequestModelId] = OMIT,
        sign_with_c_2_pa: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]:
        """
        Generate background music from one or more video files. Videos are combined in order. Optional description and style tags influence the generated music.

        Parameters
        ----------
        videos : typing.List[core.File]
            See core.File for more documentation

        output_format : typing.Optional[AllowedOutputFormats]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. So an mp3 with 22.05kHz sample rate at 32kbs is represented as mp3_22050_32. MP3 with 192kbps bitrate requires you to be subscribed to Creator tier or above. PCM with 44.1kHz sample rate requires you to be subscribed to Pro tier or above. Note that the μ-law format (sometimes written mu-law, often approximated as u-law) is commonly used for Twilio audio inputs.

        description : typing.Optional[str]
            Optional text description of the music you want. A maximum of 1000 characters is allowed.

        tags : typing.Optional[typing.List[str]]
            Optional list of style tags (e.g. ['upbeat', 'cinematic']). A maximum of 10 tags is allowed.

        model_id : typing.Optional[MusicVideoToMusicRequestModelId]
            The model to use for the generation.

        sign_with_c_2_pa : typing.Optional[bool]
            Whether to sign the generated song with C2PA. Applicable only for mp3 files.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]
            Generated audio file matching the video. Content-Type and file extension depend on the output_format parameter (default mp3).
        """
        async with self._client_wrapper.httpx_client.stream(
            "v1/music/video-to-music",
            method="POST",
            params={
                "output_format": output_format,
            },
            data={
                "description": description,
                "tags": tags,
                "model_id": model_id,
                "sign_with_c2pa": sign_with_c_2_pa,
            },
            files={
                "videos": videos,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        ) as _response:

            async def _stream() -> AsyncHttpResponse[typing.AsyncIterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", 1024) if request_options is not None else 1024
                        return AsyncHttpResponse(
                            response=_response,
                            data=(_chunk async for _chunk in _response.aiter_bytes(chunk_size=_chunk_size)),
                        )
                    await _response.aread()
                    if _response.status_code == 403:
                        raise ForbiddenError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 422:
                        raise UnprocessableEntityError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield await _stream()

    @contextlib.asynccontextmanager
    async def compose(
        self,
        *,
        output_format: typing.Optional[MusicComposeRequestOutputFormat] = None,
        prompt: typing.Optional[str] = OMIT,
        composition_plan: typing.Optional[BodyComposeMusicV1MusicPostCompositionPlan] = OMIT,
        music_length_ms: typing.Optional[int] = OMIT,
        model_id: typing.Optional[BodyComposeMusicV1MusicPostModelId] = OMIT,
        seed: typing.Optional[int] = OMIT,
        force_instrumental: typing.Optional[bool] = OMIT,
        respect_sections_durations: typing.Optional[bool] = OMIT,
        store_for_inpainting: typing.Optional[bool] = OMIT,
        sign_with_c_2_pa: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]:
        """
        Compose a song from a prompt or a composition plan.

        Parameters
        ----------
        output_format : typing.Optional[MusicComposeRequestOutputFormat]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. Use "auto" (the default) to let the API pick the best format for the selected model: mp3_44100_128 for v1 models and mp3_48000_192 for v2 models.

        prompt : typing.Optional[str]
            A simple text prompt to generate a song from. Cannot be used in conjunction with `composition_plan`.

        composition_plan : typing.Optional[BodyComposeMusicV1MusicPostCompositionPlan]
            A detailed composition plan to guide music generation. Cannot be used in conjunction with `prompt`.

        music_length_ms : typing.Optional[int]
            The length of the song to generate in milliseconds. Used only in conjunction with `prompt`. Must be between 3000ms and 600000ms. Optional - if not provided, the model will choose a length based on the prompt.

        model_id : typing.Optional[BodyComposeMusicV1MusicPostModelId]
            The model to use for the generation.

        seed : typing.Optional[int]
            Random seed to initialize the music generation process. Providing the same seed with the same parameters can help achieve more consistent results, but exact reproducibility is not guaranteed and outputs may change across system updates. Cannot be used in conjunction with prompt.

        force_instrumental : typing.Optional[bool]
            If true, guarantees that the generated song will be instrumental. If false, the song may or may not be instrumental depending on the `prompt`. Can only be used with `prompt`.

        respect_sections_durations : typing.Optional[bool]
            Controls how strictly section durations in the `composition_plan` are enforced. Only used with `composition_plan` and only applies to `music_v1`; for `music_v2` section durations are always enforced and this is ignored. When false for `music_v1`, the model may adjust individual section durations for better quality and latency, while preserving the total song duration from the plan.

        store_for_inpainting : typing.Optional[bool]
            Whether to store the generated song for inpainting.

        sign_with_c_2_pa : typing.Optional[bool]
            Whether to sign the generated song with C2PA. Applicable only for mp3 files.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]
            The generated audio file in the format specified
        """
        async with self._client_wrapper.httpx_client.stream(
            "v1/music",
            method="POST",
            params={
                "output_format": output_format,
            },
            json={
                "prompt": prompt,
                "composition_plan": convert_and_respect_annotation_metadata(
                    object_=composition_plan, annotation=BodyComposeMusicV1MusicPostCompositionPlan, direction="write"
                ),
                "music_length_ms": music_length_ms,
                "model_id": model_id,
                "seed": seed,
                "force_instrumental": force_instrumental,
                "respect_sections_durations": respect_sections_durations,
                "store_for_inpainting": store_for_inpainting,
                "sign_with_c2pa": sign_with_c_2_pa,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:

            async def _stream() -> AsyncHttpResponse[typing.AsyncIterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", 1024) if request_options is not None else 1024
                        return AsyncHttpResponse(
                            response=_response,
                            data=(_chunk async for _chunk in _response.aiter_bytes(chunk_size=_chunk_size)),
                        )
                    await _response.aread()
                    if _response.status_code == 422:
                        raise UnprocessableEntityError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield await _stream()

    @contextlib.asynccontextmanager
    async def compose_detailed(
        self,
        *,
        output_format: typing.Optional[MusicComposeDetailedRequestOutputFormat] = None,
        prompt: typing.Optional[str] = OMIT,
        composition_plan: typing.Optional[
            BodyComposeMusicWithADetailedResponseV1MusicDetailedPostCompositionPlan
        ] = OMIT,
        music_length_ms: typing.Optional[int] = OMIT,
        model_id: typing.Optional[BodyComposeMusicWithADetailedResponseV1MusicDetailedPostModelId] = OMIT,
        seed: typing.Optional[int] = OMIT,
        force_instrumental: typing.Optional[bool] = OMIT,
        respect_sections_durations: typing.Optional[bool] = OMIT,
        store_for_inpainting: typing.Optional[bool] = OMIT,
        with_timestamps: typing.Optional[bool] = OMIT,
        sign_with_c_2_pa: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]:
        """
        Compose a song from a prompt or a composition plan.

        Parameters
        ----------
        output_format : typing.Optional[MusicComposeDetailedRequestOutputFormat]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. Use "auto" (the default) to let the API pick the best format for the selected model: mp3_44100_128 for v1 models and mp3_48000_192 for v2 models.

        prompt : typing.Optional[str]
            A simple text prompt to generate a song from. Cannot be used in conjunction with `composition_plan`.

        composition_plan : typing.Optional[BodyComposeMusicWithADetailedResponseV1MusicDetailedPostCompositionPlan]
            A detailed composition plan to guide music generation. Cannot be used in conjunction with `prompt`.

        music_length_ms : typing.Optional[int]
            The length of the song to generate in milliseconds. Used only in conjunction with `prompt`. Must be between 3000ms and 600000ms. Optional - if not provided, the model will choose a length based on the prompt.

        model_id : typing.Optional[BodyComposeMusicWithADetailedResponseV1MusicDetailedPostModelId]
            The model to use for the generation.

        seed : typing.Optional[int]
            Random seed to initialize the music generation process. Providing the same seed with the same parameters can help achieve more consistent results, but exact reproducibility is not guaranteed and outputs may change across system updates. Cannot be used in conjunction with prompt.

        force_instrumental : typing.Optional[bool]
            If true, guarantees that the generated song will be instrumental. If false, the song may or may not be instrumental depending on the `prompt`. Can only be used with `prompt`.

        respect_sections_durations : typing.Optional[bool]
            Controls how strictly section durations in the `composition_plan` are enforced. Only used with `composition_plan` and only applies to `music_v1`; for `music_v2` section durations are always enforced and this is ignored. When false for `music_v1`, the model may adjust individual section durations for better quality and latency, while preserving the total song duration from the plan.

        store_for_inpainting : typing.Optional[bool]
            Whether to store the generated song for inpainting.

        with_timestamps : typing.Optional[bool]
            Whether to return the timestamps of the words in the generated song.

        sign_with_c_2_pa : typing.Optional[bool]
            Whether to sign the generated song with C2PA. Applicable only for mp3 files.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]
            Multipart/mixed response with JSON metadata and binary audio file
        """
        async with self._client_wrapper.httpx_client.stream(
            "v1/music/detailed",
            method="POST",
            params={
                "output_format": output_format,
            },
            json={
                "prompt": prompt,
                "composition_plan": convert_and_respect_annotation_metadata(
                    object_=composition_plan,
                    annotation=BodyComposeMusicWithADetailedResponseV1MusicDetailedPostCompositionPlan,
                    direction="write",
                ),
                "music_length_ms": music_length_ms,
                "model_id": model_id,
                "seed": seed,
                "force_instrumental": force_instrumental,
                "respect_sections_durations": respect_sections_durations,
                "store_for_inpainting": store_for_inpainting,
                "with_timestamps": with_timestamps,
                "sign_with_c2pa": sign_with_c_2_pa,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:

            async def _stream() -> AsyncHttpResponse[typing.AsyncIterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", 1024) if request_options is not None else 1024
                        return AsyncHttpResponse(
                            response=_response,
                            data=(_chunk async for _chunk in _response.aiter_bytes(chunk_size=_chunk_size)),
                        )
                    await _response.aread()
                    if _response.status_code == 422:
                        raise UnprocessableEntityError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield await _stream()

    @contextlib.asynccontextmanager
    async def stream(
        self,
        *,
        output_format: typing.Optional[MusicStreamRequestOutputFormat] = None,
        prompt: typing.Optional[str] = OMIT,
        composition_plan: typing.Optional[BodyStreamComposedMusicV1MusicStreamPostCompositionPlan] = OMIT,
        music_length_ms: typing.Optional[int] = OMIT,
        model_id: typing.Optional[BodyStreamComposedMusicV1MusicStreamPostModelId] = OMIT,
        seed: typing.Optional[int] = OMIT,
        force_instrumental: typing.Optional[bool] = OMIT,
        store_for_inpainting: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]:
        """
        Stream a composed song from a prompt or a composition plan.

        Parameters
        ----------
        output_format : typing.Optional[MusicStreamRequestOutputFormat]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. Use "auto" (the default) to let the API pick the best format for the selected model: mp3_44100_128 for v1 models and mp3_48000_192 for v2 models.

        prompt : typing.Optional[str]
            A simple text prompt to generate a song from. Cannot be used in conjunction with `composition_plan`.

        composition_plan : typing.Optional[BodyStreamComposedMusicV1MusicStreamPostCompositionPlan]
            A detailed composition plan to guide music generation. Cannot be used in conjunction with `prompt`.

        music_length_ms : typing.Optional[int]
            The length of the song to generate in milliseconds. Used only in conjunction with `prompt`. Must be between 3000ms and 600000ms. Optional - if not provided, the model will choose a length based on the prompt.

        model_id : typing.Optional[BodyStreamComposedMusicV1MusicStreamPostModelId]
            The model to use for the generation.

        seed : typing.Optional[int]
            Random seed to initialize the music generation process. Providing the same seed with the same parameters can help achieve more consistent results, but exact reproducibility is not guaranteed and outputs may change across system updates. Cannot be used in conjunction with prompt.

        force_instrumental : typing.Optional[bool]
            If true, guarantees that the generated song will be instrumental. If false, the song may or may not be instrumental depending on the `prompt`. Can only be used with `prompt`.

        store_for_inpainting : typing.Optional[bool]
            Whether to store the generated song for inpainting.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]
            Streaming audio data in the format specified
        """
        async with self._client_wrapper.httpx_client.stream(
            "v1/music/stream",
            method="POST",
            params={
                "output_format": output_format,
            },
            json={
                "prompt": prompt,
                "composition_plan": convert_and_respect_annotation_metadata(
                    object_=composition_plan,
                    annotation=BodyStreamComposedMusicV1MusicStreamPostCompositionPlan,
                    direction="write",
                ),
                "music_length_ms": music_length_ms,
                "model_id": model_id,
                "seed": seed,
                "force_instrumental": force_instrumental,
                "store_for_inpainting": store_for_inpainting,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:

            async def _stream() -> AsyncHttpResponse[typing.AsyncIterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", 1024) if request_options is not None else 1024
                        return AsyncHttpResponse(
                            response=_response,
                            data=(_chunk async for _chunk in _response.aiter_bytes(chunk_size=_chunk_size)),
                        )
                    await _response.aread()
                    if _response.status_code == 422:
                        raise UnprocessableEntityError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield await _stream()

    async def upload(
        self,
        *,
        file: core.File,
        extract_composition_plan: typing.Optional[str] = OMIT,
        with_timestamps: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[MusicUploadResponse]:
        """
        Upload a music file to be later used for inpainting. Price for uploading is the same as the one for song generation. All uploaded content gets inspected for copyright infringement. If copyrighted content is detected, half of the request cost is still charged.

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        extract_composition_plan : typing.Optional[str]
            Whether to generate and return the composition plan for the uploaded song. Pass a model id (`music_v1` or `music_v2`) to control which composition plan format is returned. Passing `true`/`false` is deprecated; `true` defaults to the `music_v1` plan format. Enabling this will increase the latency.

        with_timestamps : typing.Optional[bool]
            Whether to transcribe the uploaded song and return word-level timestamps. If True, the response will include words_timestamps but will increase the latency.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[MusicUploadResponse]
            Successfully uploaded music file with optional composition plan
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/music/upload",
            method="POST",
            data={
                "extract_composition_plan": extract_composition_plan,
                "with_timestamps": with_timestamps,
            },
            files={
                "file": file,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MusicUploadResponse,
                    construct_type(
                        type_=MusicUploadResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    @contextlib.asynccontextmanager
    async def separate_stems(
        self,
        *,
        file: core.File,
        output_format: typing.Optional[AllowedOutputFormats] = None,
        stem_variation_id: typing.Optional[MusicSeparateStemsRequestStemVariationId] = OMIT,
        sign_with_c_2_pa: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]:
        """
        Separate an audio file into individual stems. This endpoint might have high latency, depending on the length of the audio file.

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        output_format : typing.Optional[AllowedOutputFormats]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. So an mp3 with 22.05kHz sample rate at 32kbs is represented as mp3_22050_32. MP3 with 192kbps bitrate requires you to be subscribed to Creator tier or above. PCM with 44.1kHz sample rate requires you to be subscribed to Pro tier or above. Note that the μ-law format (sometimes written mu-law, often approximated as u-law) is commonly used for Twilio audio inputs.

        stem_variation_id : typing.Optional[MusicSeparateStemsRequestStemVariationId]
            The id of the stem variation to use.

        sign_with_c_2_pa : typing.Optional[bool]
            Whether to sign the generated song with C2PA. Applicable only for mp3 files.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]
            ZIP archive containing separated audio stems. Each stem is provided as a separate audio file in the requested output format.
        """
        async with self._client_wrapper.httpx_client.stream(
            "v1/music/stem-separation",
            method="POST",
            params={
                "output_format": output_format,
            },
            data={
                "stem_variation_id": stem_variation_id,
                "sign_with_c2pa": sign_with_c_2_pa,
            },
            files={
                "file": file,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        ) as _response:

            async def _stream() -> AsyncHttpResponse[typing.AsyncIterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", 1024) if request_options is not None else 1024
                        return AsyncHttpResponse(
                            response=_response,
                            data=(_chunk async for _chunk in _response.aiter_bytes(chunk_size=_chunk_size)),
                        )
                    await _response.aread()
                    if _response.status_code == 422:
                        raise UnprocessableEntityError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield await _stream()



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.http_response import AsyncHttpResponse, HttpResponse
from ....core.jsonable_encoder import jsonable_encoder
from ....core.request_options import RequestOptions
from ....core.serialization import convert_and_respect_annotation_metadata
from ....core.unchecked_base_model import construct_type
from ....errors.unprocessable_entity_error import UnprocessableEntityError
from ....types.item_id import ItemId
from ....types.order_id import OrderId
from ....types.remove_order_item_response import RemoveOrderItemResponse
from ....types.upsert_order_item_request import UpsertOrderItemRequest
from ....types.upsert_order_item_response import UpsertOrderItemResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawItemsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def upsert(
        self,
        order_id: OrderId,
        *,
        request: UpsertOrderItemRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpsertOrderItemResponse]:
        """
        Adds or updates an order item on an open order. Returns the item ID and the quoted price.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order.

        request : UpsertOrderItemRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpsertOrderItemResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/productions/orders/{jsonable_encoder(order_id)}/items",
            method="POST",
            json={
                "request": convert_and_respect_annotation_metadata(
                    object_=request, annotation=UpsertOrderItemRequest, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpsertOrderItemResponse,
                    construct_type(
                        type_=UpsertOrderItemResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def remove(
        self, order_id: OrderId, item_id: ItemId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[RemoveOrderItemResponse]:
        """
        Removes an order item from an open order.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order.

        item_id : ItemId
            The ID of the order item.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RemoveOrderItemResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/productions/orders/{jsonable_encoder(order_id)}/items/{jsonable_encoder(item_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RemoveOrderItemResponse,
                    construct_type(
                        type_=RemoveOrderItemResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawItemsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def upsert(
        self,
        order_id: OrderId,
        *,
        request: UpsertOrderItemRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpsertOrderItemResponse]:
        """
        Adds or updates an order item on an open order. Returns the item ID and the quoted price.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order.

        request : UpsertOrderItemRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpsertOrderItemResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/productions/orders/{jsonable_encoder(order_id)}/items",
            method="POST",
            json={
                "request": convert_and_respect_annotation_metadata(
                    object_=request, annotation=UpsertOrderItemRequest, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpsertOrderItemResponse,
                    construct_type(
                        type_=UpsertOrderItemResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def remove(
        self, order_id: OrderId, item_id: ItemId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[RemoveOrderItemResponse]:
        """
        Removes an order item from an open order.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order.

        item_id : ItemId
            The ID of the order item.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RemoveOrderItemResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/productions/orders/{jsonable_encoder(order_id)}/items/{jsonable_encoder(item_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RemoveOrderItemResponse,
                    construct_type(
                        type_=RemoveOrderItemResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel, UnionMetadata


class ProjectExternalAudioResponseModelSourceContext_MusicExploreSong(UncheckedBaseModel):
    source_type: typing.Literal["music_explore_song"] = "music_explore_song"
    music_explore_song_id: str
    title: typing.Optional[str] = None
    description: typing.Optional[str] = None
    bpm: typing.Optional[int] = None
    vocals: typing.Optional[str] = None
    lyrics: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ProjectExternalAudioResponseModelSourceContext_Sfx(UncheckedBaseModel):
    source_type: typing.Literal["sfx"] = "sfx"
    sound_generation_history_item_id: typing.Optional[str] = None
    text: typing.Optional[str] = None
    generation_config: typing.Optional[typing.Dict[str, typing.Any]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ProjectExternalAudioResponseModelSourceContext_Song(UncheckedBaseModel):
    source_type: typing.Literal["song"] = "song"
    song_id: str
    chat_id: typing.Optional[str] = None
    title: typing.Optional[str] = None
    description: typing.Optional[str] = None
    genres: typing.Optional[typing.List[str]] = None
    languages: typing.Optional[typing.List[str]] = None
    is_explicit: typing.Optional[bool] = None
    bpm: typing.Optional[int] = None
    generation_settings: typing.Optional[typing.Dict[str, typing.Any]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


ProjectExternalAudioResponseModelSourceContext = typing_extensions.Annotated[
    typing.Union[
        ProjectExternalAudioResponseModelSourceContext_MusicExploreSong,
        ProjectExternalAudioResponseModelSourceContext_Sfx,
        ProjectExternalAudioResponseModelSourceContext_Song,
    ],
    UnionMetadata(discriminant="source_type"),
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .media_codec import MediaCodec
from .sip_media_encryption_enum import SipMediaEncryptionEnum
from .sip_trunk_credentials_request_model import SipTrunkCredentialsRequestModel
from .sip_trunk_transport_enum import SipTrunkTransportEnum


class OutboundSipTrunkConfigRequestModel(UncheckedBaseModel):
    address: str = pydantic.Field()
    """
    Hostname or IP the SIP INVITE is sent to.
    """

    transport: typing.Optional[SipTrunkTransportEnum] = pydantic.Field(default=None)
    """
    Protocol to use for SIP transport (signalling layer).
    """

    media_encryption: typing.Optional[SipMediaEncryptionEnum] = pydantic.Field(default=None)
    """
    Whether or not to encrypt media (data layer).
    """

    headers: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    SIP X-* headers for INVITE request. These headers are sent as-is and may help identify this call.
    """

    attributes_to_headers: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Map of dynamic variable name to header name for attributes_to_headers
    """

    credentials: typing.Optional[SipTrunkCredentialsRequestModel] = pydantic.Field(default=None)
    """
    Optional digest authentication credentials (username/password). If not provided, ACL authentication is assumed.
    """

    enabled_codecs: typing.Optional[typing.List[MediaCodec]] = pydantic.Field(default=None)
    """
    Media codecs that should be offered in the SDP for outbound calls. If empty, all supported codecs are offered.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

DefaultSharingGroupResponseModelPermissionLevel = typing.Union[typing.Literal["admin", "editor", "viewer"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing

from .conv_ai_secret_locator import ConvAiSecretLocator
from .conv_ai_user_secret_db_model import ConvAiUserSecretDbModel

McpServerConfigInputSecretToken = typing.Union[ConvAiSecretLocator, ConvAiUserSecretDbModel]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...types.audio_output import AudioOutput
from ...types.final_output import FinalOutput

ReceiveMessage = typing.Union[AudioOutput, FinalOutput]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .audio_ref_chunk import AudioRefChunk
from .generation_chunk_input_condition_strength import GenerationChunkInputConditionStrength
from .generation_chunk_input_context_adherence import GenerationChunkInputContextAdherence


class GenerationChunkInput(UncheckedBaseModel):
    text: str = pydantic.Field()
    """
    The text config to be generated for this chunk. Can contain section name in square brackets, e.g. [Verse 1], lyrics lines, and inline directions in curly braces, e.g. {scratching}.
    """

    duration_ms: int = pydantic.Field()
    """
    The duration of the chunk in milliseconds. Must be between 3000ms and 120000ms.
    """

    positive_styles: typing.List[str] = pydantic.Field()
    """
    The styles and musical directions that should be present in this chunk. Use English language for best results. The styles for the first chunk are the most important as they set the overall tone and genre. Styles for subsequent chunks can be used to add nuance, progression, emphasis, or change the direction of the song. Aim to have at least 6-7 styles in early chunks until the direction is established. Generic styles like 'great production quality' are good default styles to append to the list.
    """

    negative_styles: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The styles and musical directions that should not be present in this chunk. Use English language for best results. Leaving empty is a good default, only use this field if you want to explicitly avoid a particular style or direction.
    """

    context_adherence: typing.Optional[GenerationChunkInputContextAdherence] = pydantic.Field(default=None)
    """
    How much the model adheres to the context of its surrounding chunks. Low adherence means the model can deviate from the context and be more creative. High adherence means the model will be more consistent with the context.
    """

    conditioning_ref: typing.Optional[AudioRefChunk] = pydantic.Field(default=None)
    """
    The audio reference to condition the generation on. The first chunk is the most important as it will influence the generation of all subsequent chunks. Thus, if you want to apply conditioning to the entire song, start conditioning from the first chunk.
    """

    condition_strength: typing.Optional[GenerationChunkInputConditionStrength] = pydantic.Field(default=None)
    """
    How strongly the model adheres to the conditioning reference. Low strength means the model will be more creative and deviate from the reference. High strength means the model will be more consistent with the reference.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawWorkspacesClient, RawWorkspacesClient

if typing.TYPE_CHECKING:
    from .api_keys.client import ApiKeysClient, AsyncApiKeysClient


class WorkspacesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawWorkspacesClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._api_keys: typing.Optional[ApiKeysClient] = None

    @property
    def with_raw_response(self) -> RawWorkspacesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawWorkspacesClient
        """
        return self._raw_client

    @property
    def api_keys(self):
        if self._api_keys is None:
            from .api_keys.client import ApiKeysClient  # noqa: E402

            self._api_keys = ApiKeysClient(client_wrapper=self._client_wrapper)
        return self._api_keys


class AsyncWorkspacesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawWorkspacesClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._api_keys: typing.Optional[AsyncApiKeysClient] = None

    @property
    def with_raw_response(self) -> AsyncRawWorkspacesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawWorkspacesClient
        """
        return self._raw_client

    @property
    def api_keys(self):
        if self._api_keys is None:
            from .api_keys.client import AsyncApiKeysClient  # noqa: E402

            self._api_keys = AsyncApiKeysClient(client_wrapper=self._client_wrapper)
        return self._api_keys



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .resource_access_info_access_source import ResourceAccessInfoAccessSource
from .resource_access_info_anonymous_access_level_override import ResourceAccessInfoAnonymousAccessLevelOverride
from .resource_access_info_role import ResourceAccessInfoRole


class ResourceAccessInfo(UncheckedBaseModel):
    is_creator: bool = pydantic.Field()
    """
    Whether the user making the request is the creator of the agent
    """

    creator_name: str = pydantic.Field()
    """
    Name of the agent's creator
    """

    creator_email: str = pydantic.Field()
    """
    Email of the agent's creator
    """

    role: ResourceAccessInfoRole = pydantic.Field()
    """
    The role of the user making the request
    """

    anonymous_access_level_override: typing.Optional[ResourceAccessInfoAnonymousAccessLevelOverride] = pydantic.Field(
        default=None
    )
    """
    The access level for anonymous users. If None, the resource is not shared publicly.
    """

    access_source: typing.Optional[ResourceAccessInfoAccessSource] = pydantic.Field(default=None)
    """
    Why the requesting user has access to this resource. 'creator' = caller is the owner. 'explicit' = caller (or one of their workspace groups) is listed in role_to_group_ids beyond the workspace-wide everyone group. 'workspace_default' = the workspace-wide everyone group is listed in role_to_group_ids (every non-anon workspace member, including admins, sees this resource). 'workspace_admin' = caller is a workspace admin and the admin seat is the *only* path to access; reserved for docs nobody else can see. Lets the UI disclose why an admin-bypass viewer sees a doc that wasn't explicitly shared with them.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .time_range import TimeRange


class AudioRefChunk(UncheckedBaseModel):
    song_id: str = pydantic.Field()
    """
    The ID of the song to source the chunk from. You can find the song ID in the response headers when you generate a song.
    """

    range: TimeRange = pydantic.Field()
    """
    The time range to extract from the song.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.unchecked_base_model import UncheckedBaseModel
from .dynamic_variable_assignment import DynamicVariableAssignment
from .dynamic_variables_config_output import DynamicVariablesConfigOutput
from .pre_tool_speech_mode import PreToolSpeechMode
from .tool_call_sound_behavior import ToolCallSoundBehavior
from .tool_call_sound_type import ToolCallSoundType
from .tool_error_handling_mode import ToolErrorHandlingMode
from .tool_execution_mode import ToolExecutionMode


class ClientToolConfigOutput(UncheckedBaseModel):
    """
    A client tool is one that sends an event to the user's client to trigger something client side
    """

    name: str
    description: str = pydantic.Field()
    """
    Description of when the tool should be used and what it does.
    """

    response_timeout_secs: typing.Optional[int] = pydantic.Field(default=None)
    """
    The maximum time in seconds to wait for the tool call to complete. Must be between 1 and 120 seconds (inclusive).
    """

    disable_interruptions: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If true, the user will not be able to interrupt the agent while this tool is running.
    """

    force_pre_tool_speech: typing.Optional[bool] = pydantic.Field(default=None)
    """
    DEPRECATED: use `pre_tool_speech` instead. If true, the agent will speak before the tool call.
    """

    pre_tool_speech: typing.Optional[PreToolSpeechMode] = pydantic.Field(default=None)
    """
    Controls whether the agent speaks before this tool is called. 'auto' (default) decides based on recent tool latency, 'force' always asks the agent to speak, 'off' fully opts out regardless of latency.
    """

    assignments: typing.Optional[typing.List[DynamicVariableAssignment]] = pydantic.Field(default=None)
    """
    Configuration for extracting values from tool responses and assigning them to dynamic variables
    """

    tool_call_sound: typing.Optional[ToolCallSoundType] = pydantic.Field(default=None)
    """
    Predefined tool call sound type to play during tool execution. If not specified, no tool call sound will be played.
    """

    tool_call_sound_behavior: typing.Optional[ToolCallSoundBehavior] = pydantic.Field(default=None)
    """
    Determines when the tool call sound should play. 'auto' only plays when there's pre-tool speech, 'always' plays for every tool call.
    """

    tool_error_handling_mode: typing.Optional[ToolErrorHandlingMode] = pydantic.Field(default=None)
    """
    Controls how tool errors are processed before being shared with the agent. 'auto' determines handling based on tool type (summarized for native integrations, hide for others), 'summarized' sends an LLM-generated summary, 'passthrough' sends the raw error, 'hide' does not share the error with the agent.
    """

    parameters: typing.Optional["ObjectJsonSchemaPropertyOutput"] = pydantic.Field(default=None)
    """
    Schema for any parameters to pass to the client
    """

    expects_response: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If true, calling this tool should block the conversation until the client responds with some response which is passed to the llm. If false then we will continue the conversation without waiting for the client to respond, this is useful to show content to a user but not block the conversation
    """

    dynamic_variables: typing.Optional[DynamicVariablesConfigOutput] = pydantic.Field(default=None)
    """
    Configuration for dynamic variables
    """

    execution_mode: typing.Optional[ToolExecutionMode] = pydantic.Field(default=None)
    """
    Determines when and how the tool executes: 'immediate' executes the tool right away when requested by the LLM, 'post_tool_speech' waits for the agent to finish speaking before executing, 'async' runs the tool in the background without blocking - best for long-running operations.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .object_json_schema_property_output import ObjectJsonSchemaPropertyOutput  # noqa: E402, I001

update_forward_refs(ClientToolConfigOutput)



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.agent_call_limits import AgentCallLimits
from ..types.agent_sort_by import AgentSortBy
from ..types.asr_conversational_config import AsrConversationalConfig
from ..types.base_turn_config import BaseTurnConfig
from ..types.conversation_config_input import ConversationConfigInput
from ..types.list_speech_engines_response import ListSpeechEnginesResponse
from ..types.privacy_config_input import PrivacyConfigInput
from ..types.sort_direction import SortDirection
from ..types.speech_engine_config import SpeechEngineConfig
from ..types.speech_engine_conversation_initiation_client_data_config import SpeechEngineConversationInitiationClientDataConfig
from ..types.speech_engine_response import SpeechEngineResponse
from ..types.tts_conversational_config_input import TtsConversationalConfigInput
from .raw_client import AsyncRawSpeechEngineClient, RawSpeechEngineClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SpeechEngineClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSpeechEngineClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSpeechEngineClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSpeechEngineClient
        """
        return self._raw_client

    def list(
        self,
        *,
        page_size: typing.Optional[int] = None,
        search: typing.Optional[str] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        sort_by: typing.Optional[AgentSortBy] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListSpeechEnginesResponse:
        """
        Returns a paginated list of Speech Engine resources.

        Parameters
        ----------
        page_size : typing.Optional[int]
            How many Speech Engines to return at maximum. Can not exceed 100, defaults to 30.

        search : typing.Optional[str]
            Search term to filter Speech Engines by name

        sort_direction : typing.Optional[SortDirection]
            The direction to sort the results

        sort_by : typing.Optional[AgentSortBy]
            The field to sort the results by

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListSpeechEnginesResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.speech_engine.list(
            page_size=1,
            search="search",
            sort_direction="asc",
            sort_by="name",
            cursor="cursor",
        )
        """
        _response = self._raw_client.list(
            page_size=page_size,
            search=search,
            sort_direction=sort_direction,
            sort_by=sort_by,
            cursor=cursor,
            request_options=request_options,
        )
        return _response.data

    def create(
        self,
        *,
        speech_engine: SpeechEngineConfig,
        name: typing.Optional[str] = OMIT,
        asr: typing.Optional[AsrConversationalConfig] = OMIT,
        tts: typing.Optional[TtsConversationalConfigInput] = OMIT,
        turn: typing.Optional[BaseTurnConfig] = OMIT,
        conversation: typing.Optional[ConversationConfigInput] = OMIT,
        privacy: typing.Optional[PrivacyConfigInput] = OMIT,
        call_limits: typing.Optional[AgentCallLimits] = OMIT,
        language: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        overrides: typing.Optional[SpeechEngineConversationInitiationClientDataConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SpeechEngineResponse:
        """
        Create a new Speech Engine resource

        Parameters
        ----------
        speech_engine : SpeechEngineConfig
            Speech engine WebSocket configuration

        name : typing.Optional[str]
            Name of the speech engine

        asr : typing.Optional[AsrConversationalConfig]
            ASR configuration

        tts : typing.Optional[TtsConversationalConfigInput]
            TTS configuration

        turn : typing.Optional[BaseTurnConfig]
            Turn detection configuration

        conversation : typing.Optional[ConversationConfigInput]
            Conversation configuration (client events, etc.)

        privacy : typing.Optional[PrivacyConfigInput]
            Privacy settings (recording, retention, zero retention mode)

        call_limits : typing.Optional[AgentCallLimits]
            Concurrency and daily conversation limits for this speech engine

        language : typing.Optional[str]
            Language for the speech engine

        tags : typing.Optional[typing.Sequence[str]]
            Tags for categorization

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SpeechEngineResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs, SpeechEngineConfig

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.speech_engine.create(
            speech_engine=SpeechEngineConfig(
                ws_url="ws_url",
            ),
        )
        """
        _response = self._raw_client.create(
            speech_engine=speech_engine,
            name=name,
            asr=asr,
            tts=tts,
            turn=turn,
            conversation=conversation,
            privacy=privacy,
            call_limits=call_limits,
            language=language,
            tags=tags,
            overrides=overrides,
            request_options=request_options,
        )
        return _response.data

    def get(
        self, speech_engine_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SpeechEngineResponse:
        """
        Retrieve a Speech Engine resource

        Parameters
        ----------
        speech_engine_id : str
            The speech engine ID (accepts seng_ or agent_ prefix)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SpeechEngineResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.speech_engine.get(
            speech_engine_id="seng_3701k3ttaq12ewp8b7qv5rfyszkz",
        )
        """
        _response = self._raw_client.get(speech_engine_id, request_options=request_options)
        return _response.data

    def delete(self, speech_engine_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a Speech Engine resource

        Parameters
        ----------
        speech_engine_id : str
            The speech engine ID (accepts seng_ or agent_ prefix)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.speech_engine.delete(
            speech_engine_id="seng_3701k3ttaq12ewp8b7qv5rfyszkz",
        )
        """
        _response = self._raw_client.delete(speech_engine_id, request_options=request_options)
        return _response.data

    def update(
        self,
        speech_engine_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        speech_engine: typing.Optional[SpeechEngineConfig] = OMIT,
        asr: typing.Optional[AsrConversationalConfig] = OMIT,
        tts: typing.Optional[TtsConversationalConfigInput] = OMIT,
        turn: typing.Optional[BaseTurnConfig] = OMIT,
        conversation: typing.Optional[ConversationConfigInput] = OMIT,
        privacy: typing.Optional[PrivacyConfigInput] = OMIT,
        call_limits: typing.Optional[AgentCallLimits] = OMIT,
        language: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SpeechEngineResponse:
        """
        Update a Speech Engine resource (partial update)

        Parameters
        ----------
        speech_engine_id : str
            The speech engine ID (accepts seng_ or agent_ prefix)

        name : typing.Optional[str]

        speech_engine : typing.Optional[SpeechEngineConfig]

        asr : typing.Optional[AsrConversationalConfig]

        tts : typing.Optional[TtsConversationalConfigInput]

        turn : typing.Optional[BaseTurnConfig]

        conversation : typing.Optional[ConversationConfigInput]

        privacy : typing.Optional[PrivacyConfigInput]

        call_limits : typing.Optional[AgentCallLimits]

        language : typing.Optional[str]

        tags : typing.Optional[typing.Sequence[str]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SpeechEngineResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.speech_engine.update(
            speech_engine_id="seng_3701k3ttaq12ewp8b7qv5rfyszkz",
        )
        """
        _response = self._raw_client.update(
            speech_engine_id,
            name=name,
            speech_engine=speech_engine,
            asr=asr,
            tts=tts,
            turn=turn,
            conversation=conversation,
            privacy=privacy,
            call_limits=call_limits,
            language=language,
            tags=tags,
            request_options=request_options,
        )
        return _response.data


class AsyncSpeechEngineClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSpeechEngineClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSpeechEngineClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSpeechEngineClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        page_size: typing.Optional[int] = None,
        search: typing.Optional[str] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        sort_by: typing.Optional[AgentSortBy] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListSpeechEnginesResponse:
        """
        Returns a paginated list of Speech Engine resources.

        Parameters
        ----------
        page_size : typing.Optional[int]
            How many Speech Engines to return at maximum. Can not exceed 100, defaults to 30.

        search : typing.Optional[str]
            Search term to filter Speech Engines by name

        sort_direction : typing.Optional[SortDirection]
            The direction to sort the results

        sort_by : typing.Optional[AgentSortBy]
            The field to sort the results by

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListSpeechEnginesResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.speech_engine.list(
                page_size=1,
                search="search",
                sort_direction="asc",
                sort_by="name",
                cursor="cursor",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            page_size=page_size,
            search=search,
            sort_direction=sort_direction,
            sort_by=sort_by,
            cursor=cursor,
            request_options=request_options,
        )
        return _response.data

    async def create(
        self,
        *,
        speech_engine: SpeechEngineConfig,
        name: typing.Optional[str] = OMIT,
        asr: typing.Optional[AsrConversationalConfig] = OMIT,
        tts: typing.Optional[TtsConversationalConfigInput] = OMIT,
        turn: typing.Optional[BaseTurnConfig] = OMIT,
        conversation: typing.Optional[ConversationConfigInput] = OMIT,
        privacy: typing.Optional[PrivacyConfigInput] = OMIT,
        call_limits: typing.Optional[AgentCallLimits] = OMIT,
        language: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        overrides: typing.Optional[SpeechEngineConversationInitiationClientDataConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SpeechEngineResponse:
        """
        Create a new Speech Engine resource

        Parameters
        ----------
        speech_engine : SpeechEngineConfig
            Speech engine WebSocket configuration

        name : typing.Optional[str]
            Name of the speech engine

        asr : typing.Optional[AsrConversationalConfig]
            ASR configuration

        tts : typing.Optional[TtsConversationalConfigInput]
            TTS configuration

        turn : typing.Optional[BaseTurnConfig]
            Turn detection configuration

        conversation : typing.Optional[ConversationConfigInput]
            Conversation configuration (client events, etc.)

        privacy : typing.Optional[PrivacyConfigInput]
            Privacy settings (recording, retention, zero retention mode)

        call_limits : typing.Optional[AgentCallLimits]
            Concurrency and daily conversation limits for this speech engine

        language : typing.Optional[str]
            Language for the speech engine

        tags : typing.Optional[typing.Sequence[str]]
            Tags for categorization

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SpeechEngineResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs, SpeechEngineConfig

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.speech_engine.create(
                speech_engine=SpeechEngineConfig(
                    ws_url="ws_url",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            speech_engine=speech_engine,
            name=name,
            asr=asr,
            tts=tts,
            turn=turn,
            conversation=conversation,
            privacy=privacy,
            call_limits=call_limits,
            language=language,
            tags=tags,
            overrides=overrides,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, speech_engine_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SpeechEngineResponse:
        """
        Retrieve a Speech Engine resource

        Parameters
        ----------
        speech_engine_id : str
            The speech engine ID (accepts seng_ or agent_ prefix)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SpeechEngineResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.speech_engine.get(
                speech_engine_id="seng_3701k3ttaq12ewp8b7qv5rfyszkz",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(speech_engine_id, request_options=request_options)
        return _response.data

    async def delete(self, speech_engine_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a Speech Engine resource

        Parameters
        ----------
        speech_engine_id : str
            The speech engine ID (accepts seng_ or agent_ prefix)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.speech_engine.delete(
                speech_engine_id="seng_3701k3ttaq12ewp8b7qv5rfyszkz",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(speech_engine_id, request_options=request_options)
        return _response.data

    async def update(
        self,
        speech_engine_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        speech_engine: typing.Optional[SpeechEngineConfig] = OMIT,
        asr: typing.Optional[AsrConversationalConfig] = OMIT,
        tts: typing.Optional[TtsConversationalConfigInput] = OMIT,
        turn: typing.Optional[BaseTurnConfig] = OMIT,
        conversation: typing.Optional[ConversationConfigInput] = OMIT,
        privacy: typing.Optional[PrivacyConfigInput] = OMIT,
        call_limits: typing.Optional[AgentCallLimits] = OMIT,
        language: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SpeechEngineResponse:
        """
        Update a Speech Engine resource (partial update)

        Parameters
        ----------
        speech_engine_id : str
            The speech engine ID (accepts seng_ or agent_ prefix)

        name : typing.Optional[str]

        speech_engine : typing.Optional[SpeechEngineConfig]

        asr : typing.Optional[AsrConversationalConfig]

        tts : typing.Optional[TtsConversationalConfigInput]

        turn : typing.Optional[BaseTurnConfig]

        conversation : typing.Optional[ConversationConfigInput]

        privacy : typing.Optional[PrivacyConfigInput]

        call_limits : typing.Optional[AgentCallLimits]

        language : typing.Optional[str]

        tags : typing.Optional[typing.Sequence[str]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SpeechEngineResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.speech_engine.update(
                speech_engine_id="seng_3701k3ttaq12ewp8b7qv5rfyszkz",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            speech_engine_id,
            name=name,
            speech_engine=speech_engine,
            asr=asr,
            tts=tts,
            turn=turn,
            conversation=conversation,
            privacy=privacy,
            call_limits=call_limits,
            language=language,
            tags=tags,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

TwilioEdgeLocation = typing.Union[
    typing.Literal[
        "ashburn", "dublin", "frankfurt", "sao-paulo", "singapore", "sydney", "tokyo", "umatilla", "roaming"
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

RttOutputFormatEnum = typing.Union[
    typing.Literal[
        "mp3_22050_32",
        "mp3_44100_32",
        "mp3_44100_64",
        "mp3_44100_96",
        "mp3_44100_128",
        "mp3_44100_192",
        "pcm_16000",
        "pcm_22050",
        "pcm_24000",
        "pcm_44100",
        "ulaw_8000",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

McpApprovalPolicy = typing.Union[
    typing.Literal["auto_approve_all", "require_approval_all", "require_approval_per_tool"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from .raw_client import AsyncRawPreviewClient, RawPreviewClient


class PreviewClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPreviewClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawPreviewClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPreviewClient
        """
        return self._raw_client

    def stream(
        self, generated_voice_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Iterator[bytes]:
        """
        Stream a voice preview that was created via the /v1/text-to-voice/design endpoint.

        Parameters
        ----------
        generated_voice_id : str
            The generated_voice_id to stream.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            Streaming audio data

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.text_to_voice.preview.stream(
            generated_voice_id="generated_voice_id",
        )
        """
        with self._raw_client.stream(generated_voice_id, request_options=request_options) as r:
            yield from r.data


class AsyncPreviewClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPreviewClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawPreviewClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPreviewClient
        """
        return self._raw_client

    async def stream(
        self, generated_voice_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.AsyncIterator[bytes]:
        """
        Stream a voice preview that was created via the /v1/text-to-voice/design endpoint.

        Parameters
        ----------
        generated_voice_id : str
            The generated_voice_id to stream.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            Streaming audio data

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.text_to_voice.preview.stream(
                generated_voice_id="generated_voice_id",
            )


        asyncio.run(main())
        """
        async with self._raw_client.stream(generated_voice_id, request_options=request_options) as r:
            async for _chunk in r.data:
                yield _chunk



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.create_phone_number_response_model import CreatePhoneNumberResponseModel
from ...types.get_sip_log_messages_response import GetSipLogMessagesResponse
from ...types.inbound_sip_trunk_config_request_model import InboundSipTrunkConfigRequestModel
from ...types.livekit_stack_type import LivekitStackType
from ...types.outbound_sip_trunk_config_request_model import OutboundSipTrunkConfigRequestModel
from ...types.telephony_provider import TelephonyProvider
from .raw_client import AsyncRawPhoneNumbersClient, RawPhoneNumbersClient
from .types.phone_numbers_create_request_body import PhoneNumbersCreateRequestBody
from .types.phone_numbers_get_response import PhoneNumbersGetResponse
from .types.phone_numbers_list_response_item import PhoneNumbersListResponseItem
from .types.phone_numbers_update_response import PhoneNumbersUpdateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PhoneNumbersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPhoneNumbersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawPhoneNumbersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPhoneNumbersClient
        """
        return self._raw_client

    def list(
        self,
        *,
        provider: typing.Optional[TelephonyProvider] = None,
        agent_id: typing.Optional[str] = None,
        branch_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[PhoneNumbersListResponseItem]:
        """
        Retrieve all Phone Numbers

        Parameters
        ----------
        provider : typing.Optional[TelephonyProvider]
            Filter by telephony provider

        agent_id : typing.Optional[str]
            Filter by assigned agent ID

        branch_id : typing.Optional[str]
            Filter by assigned branch ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[PhoneNumbersListResponseItem]
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.phone_numbers.list(
            provider="twilio",
            agent_id="agent_id",
            branch_id="branch_id",
        )
        """
        _response = self._raw_client.list(
            provider=provider, agent_id=agent_id, branch_id=branch_id, request_options=request_options
        )
        return _response.data

    def create(
        self, *, request: PhoneNumbersCreateRequestBody, request_options: typing.Optional[RequestOptions] = None
    ) -> CreatePhoneNumberResponseModel:
        """
        Import Phone Number from provider configuration (Twilio, Exotel, or SIP trunk)

        Parameters
        ----------
        request : PhoneNumbersCreateRequestBody

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreatePhoneNumberResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs
        from elevenlabs.conversational_ai.phone_numbers import (
            PhoneNumbersCreateRequestBody_Twilio,
        )

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.phone_numbers.create(
            request=PhoneNumbersCreateRequestBody_Twilio(
                phone_number="phone_number",
                label="label",
                sid="sid",
                token="token",
            ),
        )
        """
        _response = self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    def get(
        self, phone_number_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PhoneNumbersGetResponse:
        """
        Retrieve Phone Number details by ID

        Parameters
        ----------
        phone_number_id : str
            The phone number ID. This is returned when a phone number is imported.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PhoneNumbersGetResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.phone_numbers.get(
            phone_number_id="TeaqRRdTcIfIu2i7BYfT",
        )
        """
        _response = self._raw_client.get(phone_number_id, request_options=request_options)
        return _response.data

    def delete(self, phone_number_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> typing.Any:
        """
        Delete Phone Number by ID

        Parameters
        ----------
        phone_number_id : str
            The phone number ID. This is returned when a phone number is imported.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Any
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.phone_numbers.delete(
            phone_number_id="TeaqRRdTcIfIu2i7BYfT",
        )
        """
        _response = self._raw_client.delete(phone_number_id, request_options=request_options)
        return _response.data

    def update(
        self,
        phone_number_id: str,
        *,
        agent_id: typing.Optional[str] = OMIT,
        label: typing.Optional[str] = OMIT,
        inbound_trunk_config: typing.Optional[InboundSipTrunkConfigRequestModel] = OMIT,
        outbound_trunk_config: typing.Optional[OutboundSipTrunkConfigRequestModel] = OMIT,
        livekit_stack: typing.Optional[LivekitStackType] = OMIT,
        store_sip_messages: typing.Optional[bool] = OMIT,
        environment: typing.Optional[str] = OMIT,
        branch_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PhoneNumbersUpdateResponse:
        """
        Update assigned agent of a phone number

        Parameters
        ----------
        phone_number_id : str
            The phone number ID. This is returned when a phone number is imported.

        agent_id : typing.Optional[str]

        label : typing.Optional[str]

        inbound_trunk_config : typing.Optional[InboundSipTrunkConfigRequestModel]

        outbound_trunk_config : typing.Optional[OutboundSipTrunkConfigRequestModel]

        livekit_stack : typing.Optional[LivekitStackType]

        store_sip_messages : typing.Optional[bool]

        environment : typing.Optional[str]
            Environment to use for resolving environment variables on calls to this number.

        branch_id : typing.Optional[str]
            Agent branch to use for calls to this number.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PhoneNumbersUpdateResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.phone_numbers.update(
            phone_number_id="TeaqRRdTcIfIu2i7BYfT",
        )
        """
        _response = self._raw_client.update(
            phone_number_id,
            agent_id=agent_id,
            label=label,
            inbound_trunk_config=inbound_trunk_config,
            outbound_trunk_config=outbound_trunk_config,
            livekit_stack=livekit_stack,
            store_sip_messages=store_sip_messages,
            environment=environment,
            branch_id=branch_id,
            request_options=request_options,
        )
        return _response.data

    def get_sip_messages(
        self,
        phone_number_id: str,
        *,
        page_size: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetSipLogMessagesResponse:
        """
        Get SIP messages for a phone number

        Parameters
        ----------
        phone_number_id : str
            The phone number ID. This is returned when a phone number is imported.

        page_size : typing.Optional[int]

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSipLogMessagesResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.phone_numbers.get_sip_messages(
            phone_number_id="TeaqRRdTcIfIu2i7BYfT",
            page_size=1,
            cursor="cursor",
        )
        """
        _response = self._raw_client.get_sip_messages(
            phone_number_id, page_size=page_size, cursor=cursor, request_options=request_options
        )
        return _response.data


class AsyncPhoneNumbersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPhoneNumbersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawPhoneNumbersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPhoneNumbersClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        provider: typing.Optional[TelephonyProvider] = None,
        agent_id: typing.Optional[str] = None,
        branch_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[PhoneNumbersListResponseItem]:
        """
        Retrieve all Phone Numbers

        Parameters
        ----------
        provider : typing.Optional[TelephonyProvider]
            Filter by telephony provider

        agent_id : typing.Optional[str]
            Filter by assigned agent ID

        branch_id : typing.Optional[str]
            Filter by assigned branch ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[PhoneNumbersListResponseItem]
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.phone_numbers.list(
                provider="twilio",
                agent_id="agent_id",
                branch_id="branch_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            provider=provider, agent_id=agent_id, branch_id=branch_id, request_options=request_options
        )
        return _response.data

    async def create(
        self, *, request: PhoneNumbersCreateRequestBody, request_options: typing.Optional[RequestOptions] = None
    ) -> CreatePhoneNumberResponseModel:
        """
        Import Phone Number from provider configuration (Twilio, Exotel, or SIP trunk)

        Parameters
        ----------
        request : PhoneNumbersCreateRequestBody

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreatePhoneNumberResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs
        from elevenlabs.conversational_ai.phone_numbers import (
            PhoneNumbersCreateRequestBody_Twilio,
        )

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.phone_numbers.create(
                request=PhoneNumbersCreateRequestBody_Twilio(
                    phone_number="phone_number",
                    label="label",
                    sid="sid",
                    token="token",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    async def get(
        self, phone_number_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PhoneNumbersGetResponse:
        """
        Retrieve Phone Number details by ID

        Parameters
        ----------
        phone_number_id : str
            The phone number ID. This is returned when a phone number is imported.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PhoneNumbersGetResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.phone_numbers.get(
                phone_number_id="TeaqRRdTcIfIu2i7BYfT",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(phone_number_id, request_options=request_options)
        return _response.data

    async def delete(
        self, phone_number_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Any:
        """
        Delete Phone Number by ID

        Parameters
        ----------
        phone_number_id : str
            The phone number ID. This is returned when a phone number is imported.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Any
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.phone_numbers.delete(
                phone_number_id="TeaqRRdTcIfIu2i7BYfT",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(phone_number_id, request_options=request_options)
        return _response.data

    async def update(
        self,
        phone_number_id: str,
        *,
        agent_id: typing.Optional[str] = OMIT,
        label: typing.Optional[str] = OMIT,
        inbound_trunk_config: typing.Optional[InboundSipTrunkConfigRequestModel] = OMIT,
        outbound_trunk_config: typing.Optional[OutboundSipTrunkConfigRequestModel] = OMIT,
        livekit_stack: typing.Optional[LivekitStackType] = OMIT,
        store_sip_messages: typing.Optional[bool] = OMIT,
        environment: typing.Optional[str] = OMIT,
        branch_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PhoneNumbersUpdateResponse:
        """
        Update assigned agent of a phone number

        Parameters
        ----------
        phone_number_id : str
            The phone number ID. This is returned when a phone number is imported.

        agent_id : typing.Optional[str]

        label : typing.Optional[str]

        inbound_trunk_config : typing.Optional[InboundSipTrunkConfigRequestModel]

        outbound_trunk_config : typing.Optional[OutboundSipTrunkConfigRequestModel]

        livekit_stack : typing.Optional[LivekitStackType]

        store_sip_messages : typing.Optional[bool]

        environment : typing.Optional[str]
            Environment to use for resolving environment variables on calls to this number.

        branch_id : typing.Optional[str]
            Agent branch to use for calls to this number.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PhoneNumbersUpdateResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.phone_numbers.update(
                phone_number_id="TeaqRRdTcIfIu2i7BYfT",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            phone_number_id,
            agent_id=agent_id,
            label=label,
            inbound_trunk_config=inbound_trunk_config,
            outbound_trunk_config=outbound_trunk_config,
            livekit_stack=livekit_stack,
            store_sip_messages=store_sip_messages,
            environment=environment,
            branch_id=branch_id,
            request_options=request_options,
        )
        return _response.data

    async def get_sip_messages(
        self,
        phone_number_id: str,
        *,
        page_size: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetSipLogMessagesResponse:
        """
        Get SIP messages for a phone number

        Parameters
        ----------
        phone_number_id : str
            The phone number ID. This is returned when a phone number is imported.

        page_size : typing.Optional[int]

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSipLogMessagesResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.phone_numbers.get_sip_messages(
                phone_number_id="TeaqRRdTcIfIu2i7BYfT",
                page_size=1,
                cursor="cursor",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_sip_messages(
            phone_number_id, page_size=page_size, cursor=cursor, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .dialogue_input_response_model import DialogueInputResponseModel
from .feedback_item import FeedbackItem
from .history_alignments_response_model import HistoryAlignmentsResponseModel
from .speech_history_item_response_model_source import SpeechHistoryItemResponseModelSource
from .speech_history_item_response_model_voice_category import SpeechHistoryItemResponseModelVoiceCategory


class SpeechHistoryItemResponse(UncheckedBaseModel):
    history_item_id: str = pydantic.Field()
    """
    The ID of the history item.
    """

    request_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the request.
    """

    voice_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the voice used.
    """

    model_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the model.
    """

    voice_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the voice.
    """

    voice_category: typing.Optional[SpeechHistoryItemResponseModelVoiceCategory] = pydantic.Field(default=None)
    """
    The category of the voice. Either 'premade', 'cloned', 'generated' or 'professional'.
    """

    text: typing.Optional[str] = pydantic.Field(default=None)
    """
    The text used to generate the audio item.
    """

    date_unix: int = pydantic.Field()
    """
    Unix timestamp of when the item was created.
    """

    character_count_change_from: int = pydantic.Field()
    """
    The character count change from.
    """

    character_count_change_to: int = pydantic.Field()
    """
    The character count change to.
    """

    content_type: str = pydantic.Field()
    """
    The content type of the generated item.
    """

    state: typing.Any
    settings: typing.Optional[typing.Dict[str, typing.Any]] = pydantic.Field(default=None)
    """
    The settings of the history item.
    """

    feedback: typing.Optional[FeedbackItem] = pydantic.Field(default=None)
    """
    Feedback associated with the generated item. Returns null if no feedback has been provided.
    """

    share_link_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the share link.
    """

    source: typing.Optional[SpeechHistoryItemResponseModelSource] = pydantic.Field(default=None)
    """
    The source of the history item. Either TTS (text to speech), STS (speech to text), AN (audio native), Projects, Dubbing, PlayAPI, PD (pronunciation dictionary) or ConvAI (Agents Platform).
    """

    alignments: typing.Optional[HistoryAlignmentsResponseModel] = pydantic.Field(default=None)
    """
    The alignments of the history item.
    """

    dialogue: typing.Optional[typing.List[DialogueInputResponseModel]] = pydantic.Field(default=None)
    """
    The dialogue (voice and text pairs) used to generate the audio item. If this is set then the top level `text` and `voice_id` fields will be empty.
    """

    output_format: typing.Optional[str] = pydantic.Field(default=None)
    """
    The output format the audio was originally generated in.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .transcription_word_type import TranscriptionWordType


class TranscriptionWord(UncheckedBaseModel):
    """
    Word-level transcription data with timing information.
    """

    text: typing.Optional[str] = pydantic.Field(default=None)
    """
    The transcribed word.
    """

    start: typing.Optional[float] = pydantic.Field(default=None)
    """
    Start time in seconds.
    """

    end: typing.Optional[float] = pydantic.Field(default=None)
    """
    End time in seconds.
    """

    type: typing.Optional[TranscriptionWordType] = pydantic.Field(default=None)
    """
    The type of word.
    """

    speaker_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the speaker if available.
    """

    logprob: typing.Optional[float] = pydantic.Field(default=None)
    """
    Confidence score for this word.
    """

    characters: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The characters in the word.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from elevenlabs import Model
from elevenlabs.client import ElevenLabs


def test_models_get_all():
    client = ElevenLabs()
    models = client.models.list()
    assert len(models) > 0
    assert isinstance(models[0], Model)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .fine_tuning_response_model_state_value import FineTuningResponseModelStateValue
from .manual_verification_response import ManualVerificationResponse
from .verification_attempt_response import VerificationAttemptResponse


class FineTuningResponse(UncheckedBaseModel):
    is_allowed_to_fine_tune: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the user is allowed to fine-tune the voice.
    """

    state: typing.Optional[typing.Dict[str, FineTuningResponseModelStateValue]] = pydantic.Field(default=None)
    """
    The state of the fine-tuning process for each model.
    """

    verification_failures: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of verification failures in the fine-tuning process.
    """

    verification_attempts_count: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of verification attempts in the fine-tuning process.
    """

    manual_verification_requested: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether a manual verification was requested for the fine-tuning process.
    """

    language: typing.Optional[str] = pydantic.Field(default=None)
    """
    The language of the fine-tuning process.
    """

    progress: typing.Optional[typing.Dict[str, typing.Optional[float]]] = pydantic.Field(default=None)
    """
    The progress of the fine-tuning process.
    """

    message: typing.Optional[typing.Dict[str, typing.Optional[str]]] = pydantic.Field(default=None)
    """
    The message of the fine-tuning process.
    """

    dataset_duration_seconds: typing.Optional[float] = pydantic.Field(default=None)
    """
    The duration of the dataset in seconds.
    """

    verification_attempts: typing.Optional[typing.List[VerificationAttemptResponse]] = pydantic.Field(default=None)
    """
    The number of verification attempts.
    """

    slice_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of slice IDs.
    """

    manual_verification: typing.Optional[ManualVerificationResponse] = pydantic.Field(default=None)
    """
    The manual verification of the fine-tuning process.
    """

    max_verification_attempts: typing.Optional[int] = pydantic.Field(default=None)
    """
    The maximum number of verification attempts.
    """

    next_max_verification_attempts_reset_unix_ms: typing.Optional[int] = pydantic.Field(default=None)
    """
    The next maximum verification attempts reset time in Unix milliseconds.
    """

    finetuning_state: typing.Optional[typing.Any] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .order_id import OrderId
from .order_state import OrderState


class SubmitOrderResponse(UncheckedBaseModel):
    order_id: OrderId = pydantic.Field()
    """
    The ID of the submitted order.
    """

    state: OrderState = pydantic.Field()
    """
    The current state of the order after submission.
    """

    submitted_at: dt.datetime = pydantic.Field()
    """
    The timestamp when the order was submitted.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .phone_number_agent_info import PhoneNumberAgentInfo


class GetPhoneNumberTwilioResponseModel(UncheckedBaseModel):
    phone_number: str = pydantic.Field()
    """
    Phone number
    """

    label: str = pydantic.Field()
    """
    Label for the phone number
    """

    supports_inbound: typing.Optional[bool] = pydantic.Field(default=None)
    """
    This field is deprecated and will be removed in the future. Whether this phone number supports inbound calls
    """

    supports_outbound: typing.Optional[bool] = pydantic.Field(default=None)
    """
    This field is deprecated and will be removed in the future. Whether this phone number supports outbound calls
    """

    phone_number_id: str = pydantic.Field()
    """
    The ID of the phone number
    """

    assigned_agent: typing.Optional[PhoneNumberAgentInfo] = pydantic.Field(default=None)
    """
    The agent that is assigned to the phone number
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .async_conversation_metadata_delivery_status import AsyncConversationMetadataDeliveryStatus


class AsyncConversationMetadata(UncheckedBaseModel):
    """
    Metadata for async conversation delivery (Zendesk, Slack, etc.).
    """

    delivery_status: AsyncConversationMetadataDeliveryStatus
    delivery_timestamp: int
    delivery_error: typing.Optional[str] = None
    external_system: str
    external_id: str
    external_link: typing.Optional[str] = None
    retry_count: typing.Optional[int] = None
    last_retry_timestamp: typing.Optional[int] = None
    last_processed_external_message_id: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.request_options import RequestOptions
from ...core.unchecked_base_model import construct_type
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.workspace_audit_logs_page_response import WorkspaceAuditLogsPageResponse


class RawAuditLogsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        time_from_unix_ms: typing.Optional[int] = None,
        time_to_unix_ms: typing.Optional[int] = None,
        actor_uid: typing.Optional[str] = None,
        class_name: typing.Optional[str] = None,
        activity_name: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[WorkspaceAuditLogsPageResponse]:
        """
        Returns the audit log for the workspace. Requires enterprise tier and the audit_log_read permission.

        Parameters
        ----------
        limit : typing.Optional[int]
            Maximum number of entries per page

        cursor : typing.Optional[str]
            Cursor for the next page (from previous response)

        time_from_unix_ms : typing.Optional[int]
            Only include entries at or after this time (ms since epoch)

        time_to_unix_ms : typing.Optional[int]
            Only include entries at or before this time (ms since epoch)

        actor_uid : typing.Optional[str]
            Filter by actor user ID

        class_name : typing.Optional[str]
            Filter by OCSF event class name (e.g. Account Change)

        activity_name : typing.Optional[str]
            Filter by audit activity name (e.g. Subscription Creation)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[WorkspaceAuditLogsPageResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/workspace/audit-logs",
            method="GET",
            params={
                "limit": limit,
                "cursor": cursor,
                "time_from_unix_ms": time_from_unix_ms,
                "time_to_unix_ms": time_to_unix_ms,
                "actor_uid": actor_uid,
                "class_name": class_name,
                "activity_name": activity_name,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WorkspaceAuditLogsPageResponse,
                    construct_type(
                        type_=WorkspaceAuditLogsPageResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawAuditLogsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        time_from_unix_ms: typing.Optional[int] = None,
        time_to_unix_ms: typing.Optional[int] = None,
        actor_uid: typing.Optional[str] = None,
        class_name: typing.Optional[str] = None,
        activity_name: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[WorkspaceAuditLogsPageResponse]:
        """
        Returns the audit log for the workspace. Requires enterprise tier and the audit_log_read permission.

        Parameters
        ----------
        limit : typing.Optional[int]
            Maximum number of entries per page

        cursor : typing.Optional[str]
            Cursor for the next page (from previous response)

        time_from_unix_ms : typing.Optional[int]
            Only include entries at or after this time (ms since epoch)

        time_to_unix_ms : typing.Optional[int]
            Only include entries at or before this time (ms since epoch)

        actor_uid : typing.Optional[str]
            Filter by actor user ID

        class_name : typing.Optional[str]
            Filter by OCSF event class name (e.g. Account Change)

        activity_name : typing.Optional[str]
            Filter by audit activity name (e.g. Subscription Creation)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[WorkspaceAuditLogsPageResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/workspace/audit-logs",
            method="GET",
            params={
                "limit": limit,
                "cursor": cursor,
                "time_from_unix_ms": time_from_unix_ms,
                "time_to_unix_ms": time_to_unix_ms,
                "actor_uid": actor_uid,
                "class_name": class_name,
                "activity_name": activity_name,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WorkspaceAuditLogsPageResponse,
                    construct_type(
                        type_=WorkspaceAuditLogsPageResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .open_ai_audio_config import OpenAiAudioConfig
from .open_ai_session_config_max_response_output_tokens import OpenAiSessionConfigMaxResponseOutputTokens
from .open_ai_session_config_tool_choice import OpenAiSessionConfigToolChoice
from .open_ai_session_config_tools_item import OpenAiSessionConfigToolsItem


class OpenAiSessionConfig(UncheckedBaseModel):
    type: typing.Optional[typing.Literal["realtime"]] = None
    model: typing.Optional[str] = None
    instructions: typing.Optional[str] = None
    modalities: typing.Optional[typing.List[str]] = None
    audio: typing.Optional[OpenAiAudioConfig] = None
    tools: typing.Optional[typing.List[OpenAiSessionConfigToolsItem]] = None
    tool_choice: typing.Optional[OpenAiSessionConfigToolChoice] = None
    temperature: typing.Optional[float] = None
    max_response_output_tokens: typing.Optional[OpenAiSessionConfigMaxResponseOutputTokens] = None
    output_modalities: typing.Optional[typing.List[str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import contextlib
import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.request_options import RequestOptions
from ..core.unchecked_base_model import construct_type
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.allowed_output_formats import AllowedOutputFormats

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawTextToSoundEffectsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    @contextlib.contextmanager
    def convert(
        self,
        *,
        text: str,
        output_format: typing.Optional[AllowedOutputFormats] = None,
        loop: typing.Optional[bool] = OMIT,
        duration_seconds: typing.Optional[float] = OMIT,
        prompt_influence: typing.Optional[float] = OMIT,
        model_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[HttpResponse[typing.Iterator[bytes]]]:
        """
        Turn text into sound effects for your videos, voice-overs or video games using the most advanced sound effects models in the world.

        Parameters
        ----------
        text : str
            The text that will get converted into a sound effect.

        output_format : typing.Optional[AllowedOutputFormats]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. So an mp3 with 22.05kHz sample rate at 32kbs is represented as mp3_22050_32. MP3 with 192kbps bitrate requires you to be subscribed to Creator tier or above. PCM with 44.1kHz sample rate requires you to be subscribed to Pro tier or above. Note that the μ-law format (sometimes written mu-law, often approximated as u-law) is commonly used for Twilio audio inputs.

        loop : typing.Optional[bool]
            Whether to create a sound effect that loops smoothly. Only available for the 'eleven_text_to_sound_v2 model'.

        duration_seconds : typing.Optional[float]
            The duration of the sound which will be generated in seconds. Must be at least 0.5 and at most 30. If set to None we will guess the optimal duration using the prompt. Defaults to None.

        prompt_influence : typing.Optional[float]
            A higher prompt influence makes your generation follow the prompt more closely while also making generations less variable. Must be a value between 0 and 1. Defaults to 0.3.

        model_id : typing.Optional[str]
            The model ID to use for the sound generation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[HttpResponse[typing.Iterator[bytes]]]
            The generated sound effect as an MP3 file
        """
        with self._client_wrapper.httpx_client.stream(
            "v1/sound-generation",
            method="POST",
            params={
                "output_format": output_format,
            },
            json={
                "text": text,
                "loop": loop,
                "duration_seconds": duration_seconds,
                "prompt_influence": prompt_influence,
                "model_id": model_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:

            def _stream() -> HttpResponse[typing.Iterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", 1024) if request_options is not None else 1024
                        return HttpResponse(
                            response=_response, data=(_chunk for _chunk in _response.iter_bytes(chunk_size=_chunk_size))
                        )
                    _response.read()
                    if _response.status_code == 422:
                        raise UnprocessableEntityError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield _stream()


class AsyncRawTextToSoundEffectsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    @contextlib.asynccontextmanager
    async def convert(
        self,
        *,
        text: str,
        output_format: typing.Optional[AllowedOutputFormats] = None,
        loop: typing.Optional[bool] = OMIT,
        duration_seconds: typing.Optional[float] = OMIT,
        prompt_influence: typing.Optional[float] = OMIT,
        model_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]:
        """
        Turn text into sound effects for your videos, voice-overs or video games using the most advanced sound effects models in the world.

        Parameters
        ----------
        text : str
            The text that will get converted into a sound effect.

        output_format : typing.Optional[AllowedOutputFormats]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. So an mp3 with 22.05kHz sample rate at 32kbs is represented as mp3_22050_32. MP3 with 192kbps bitrate requires you to be subscribed to Creator tier or above. PCM with 44.1kHz sample rate requires you to be subscribed to Pro tier or above. Note that the μ-law format (sometimes written mu-law, often approximated as u-law) is commonly used for Twilio audio inputs.

        loop : typing.Optional[bool]
            Whether to create a sound effect that loops smoothly. Only available for the 'eleven_text_to_sound_v2 model'.

        duration_seconds : typing.Optional[float]
            The duration of the sound which will be generated in seconds. Must be at least 0.5 and at most 30. If set to None we will guess the optimal duration using the prompt. Defaults to None.

        prompt_influence : typing.Optional[float]
            A higher prompt influence makes your generation follow the prompt more closely while also making generations less variable. Must be a value between 0 and 1. Defaults to 0.3.

        model_id : typing.Optional[str]
            The model ID to use for the sound generation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]
            The generated sound effect as an MP3 file
        """
        async with self._client_wrapper.httpx_client.stream(
            "v1/sound-generation",
            method="POST",
            params={
                "output_format": output_format,
            },
            json={
                "text": text,
                "loop": loop,
                "duration_seconds": duration_seconds,
                "prompt_influence": prompt_influence,
                "model_id": model_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:

            async def _stream() -> AsyncHttpResponse[typing.AsyncIterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", 1024) if request_options is not None else 1024
                        return AsyncHttpResponse(
                            response=_response,
                            data=(_chunk async for _chunk in _response.aiter_bytes(chunk_size=_chunk_size)),
                        )
                    await _response.aread()
                    if _response.status_code == 422:
                        raise UnprocessableEntityError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield await _stream()
